%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Workbook6.m is the main code file. Run it section by section to perform
% the following calculations and plot graphs:
% %% Clean data and save CSV as .mat file
%    - reads spreadsheet gridwatch_2017.csv,
%      cleans data, saves into results file gridCarbonPaper.mat
% %% Apply adapted Hawkes' method to 2017 data
%    - saves data to results file gridCarbonPaper.mat
%    - writes data to results spreadsheet GridCarbonResults.xls
%    - plots Rsq_gradients.eps (Fig. 3)
%            MGRs2017.eps (Fig. 4)
%            MEFs2017.eps (Fig. 5)
% %% C-rate testing - Arbitrage Algorithm- home PV+batt benefit
%    - saves data to results file gridCarbonPaper.mat
%    - writes data to results spreadsheet GridCarbonResults.xls
%    - plots Crate_tests.eps (Fig. 6)
% %% PV/batt sizing - Arbitrage Algorithm
%    - saves data to results file gridCarbonPaper.mat
%    - writes data to results spreadsheet GridCarbonResults.xls
%    - plots genCR50.eps (Fig. 7)
%            MEFsCR50.eps (Fig. 8)
%            PVbattsizing.eps (Fig. 9)
% %% Make movie
%    - saves video as algorithm_movie.avi
%    - plots pvbatt_aug13.eps (Fig. 2)
%
% Workbook6.m calls the following functions, which should all be in the
% same folder:
% - readGridWatch(filename, startRow, endRow)
%     reads .csv of generation data into Matlab data table in workspace.
% - coarsen_data(output,year0,Dt)
%     averages generation data from year0 (2017) to resolution Dt (1/2-h)
%     outputs arrays for each generation type, as well as time array t2,
%     and demand array Pd_.
%     [t2,bio_,nuc_,coal_,ccgt_,wind_,french_,dutch_,scot_,north_,...
%      irish_,ew_,hydro_,pumped_,solar_,Pd_]
% - hawkes_marg2(Dt,Pd_,wind_,nuc_,bio_,hydro_,dutch_,french_,coal_,ccgt_)
%     applies adapted Hawkes' method to demand and generation data input,
%     outputs marginal generator response MGR(t) for coal, CCCGT,
%     biomass, hydro, French and Dutch interconnectors.
%     [mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du,diags]
%     Also diagnostics 'diags', linear fitting R^2 values.
% - xls_2017(fname,co2_c,co2_cc,co2_b,co2_fr,co2_du,rsq_c,rsq_cc,rsq_b,...
%         rsq_h,rsq_fr,rsq_du,t_17,mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du)
%     writes MGR(t) data for 2017 into spreadsheet GridCarbonResults.xls
% - init(house,gridmix,alg,tariff,P_PV,E_B)
%     initialises model parameters: house = 4 (NOT PROVIDED due to data
%     privacy), gridmix = MGR(t) data, alg = algorithm (not used), tariff =
%     electricity bill tariff (not used), P_PV = PV capacity (kW), E_B =
%     battery capacity (kWh). Saves all parameters calculated from these
%     into the cell array model_params, which are input to...
% - run_alg3(Pgen0,Pload0,base_params,B_params,co2,Crate)
%     runs Emissions Arbitrage Algorithm using parameters input. Pgen0 =
%     normalised PV generation series, Pload0 = electricity consumption
%     series, base_params = system parameters, B_params = battery
%     parameters, co2 = MEF(t), Crate = battery power C-rate limit. Outputs
%     [Pgen,Pd,Pcp,Pcm,Pgp,Pgm,Esoc] - time series of PV, load, charging
%     power, discharging power, export power, import power, energy stored.
% - dispatch_new(Dt,Pd,w,pv,pvbatt,t,colors,facs,plotflag)
%     given time series of demand Pd, wind w, PV pv, PV-battery grid export
%     pvbatt, and system parameters facs (see .m file to unpack them),
%     dispatch generators in order of CO2 intensity to meet demand at each
%     given instant. If plotflag==true, plot genCR50.eps (Fig. 7)
% - xls_Crates(fname,sname,snames,Crates,PVbatt,PVbatt_ave,PVonly,none)
%     writes data for C-rate limit testing in 2017 and future scenarios
%     into spreadsheet GridCarbonResults.xls
% - xls_PVbattsizing(fname,snames,P_PV,E_B,results,PVbatt_17_n,...
%                 PVbatt_cr30_n,PVbatt_td30_n,PVbatt_cr50_n,PVbatt_td50_n)
%     writes data for PV/battery size testing in 2017 and future scenarios
%     into spreadsheet GridCarbonResults.xls
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%