function [Pn1,Pcc1,Pccs1,Phy1,Pb1,Pfr1,Pdu1,Pv2g,Pstor,other]=...
                 dispatch_new(Dt,Pd,w,pv,pvbatt,t,colors,facs,plotflag)
% [Pn1,Pcc1,Pccs1,Phy1,Pb1,Pfr1,Pdu1,Pv2g,Pstor,other]=...
%                  dispatch_new(Dt,Pd,w,pv,pvbatt,t,colors,facs,plotflag)
%     given time series of demand Pd, wind w, PV pv, PV-battery grid export
%     pvbatt, and system parameters facs (see .m file to unpack them),
%     dispatch generators in order of CO2 intensity to meet demand at each
%     given instant. If plotflag==true, plot genCR50.eps (Fig. 7)

N = length(Pd);
eta = sqrt(0.85); % efficiency of storage
Pd = (Pd+pv)*facs(2)+1e3*facs(1); % demand (MW)
w = w*facs(3); % scale up wind
pv = pv*facs(4); % scale up PV
nlim = facs(5); % max nuclear capacity (MW)
Pn1 = 0.9*nlim*ones(1,N); % nuc is constant at mean level
blim = facs(6); % max biomass capacity (MW)
hylim = facs(7); % max hydro capacity (MW)
cclim = facs(8); % max CCGT capacity (MW)
ccslim = facs(9); % max CCS capacity (MW)
ictlim = facs(10); % max interconnector capacity (MW)
v2glim = facs(11); % max V2G charge/discharge capacity (MW)
v2gTlim = facs(12)*facs(11); % max V2G capacity (MWh)
storPlim = facs(13); % max other storage charge/discharge capacity (MW)
storTlim = facs(14)*facs(13); % max other storage capacity (MWh)
phi1 = facs(15);
phi2 = phi1*(sum(pv)/sum(Pd));
Pd = Pd*(1-phi2)-pv*(1-phi1)+pvbatt; % demand (MW)

Phy1 = zeros(1,N);
Pcc1 = zeros(1,N);
Pccs1 = zeros(1,N);
Pb1 = zeros(1,N);
Pfr1 = zeros(1,N);
Pdu1 = zeros(1,N);
Pv2g = zeros(1,N);
Ev2g = zeros(1,N);
Ev2g(1) = v2gTlim;
Pstor = zeros(1,N);
Estor = zeros(1,N);
Estor(1) = storTlim;
other = zeros(1,N);

for nt = 1:N
  if nt>1
    Pn1(nt) = min([Pn1(nt-1)+nlim*0.02,nlim]); % keep nuclear continuity 
    Ev2g(nt) = Ev2g(nt-1);  
    Estor(nt) = Estor(nt-1);
  end
  fill = Pn1(nt)+w(nt);
  if fill > Pd(nt) % too much energy, charge up EVs smartly
%     if Ev2g(nt) < v2gTlim % EVs can accept charge
      Pv2g(nt) = max([-(fill-Pd(nt)),-v2glim*(1-Ev2g(nt)/v2gTlim),...
                                             -(v2gTlim-Ev2g(nt))/Dt/eta]);
      fill = Pn1(nt)+w(nt)+Pv2g(nt);
      Ev2g(nt) = Ev2g(nt)-eta*Pv2g(nt)*Dt;
%     end
  end
  if fill > Pd(nt) % too much energy, need to store
%     if Estor(nt) < storTlim % storage can accept charge
      Pstor(nt) = max([-(fill-Pd(nt)),-storPlim*(1-Estor(nt)/storTlim),...
                                         -(storTlim-Estor(nt))/Dt/eta]);
      fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt); 
      Estor(nt) = Estor(nt)-eta*Pstor(nt)*Dt; 
  end
%   end
  if fill > Pd(nt) % still too much energy, need to export
    Pfr1(nt) = max([-(fill-Pd(nt)),-ictlim]);
    fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt); 
  end
  if nt>1
   if fill > Pd(nt) % still too much energy, need to reduce nuclear gen
    Pn1(nt) = max([0,Pn1(nt-1)-nlim*0.02,Pn1(nt-1)-(fill-Pd(nt))]);
    fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt); 
   end
  end
  
  if fill < Pd(nt) % fill up on hydro
    Phy1(nt) = min([hylim,Pd(nt)-fill]);
    fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt);
  end
  if fill < Pd(nt) % fill up on CCS
    Pccs1(nt) = min([ccslim,Pd(nt)-fill]);
    fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt)+Pccs1(nt);
  end
  if fill < Pd(nt) % fill up from store
%     if Estor(nt) > 0 % storage not empty
     Pstor(nt) = min([(Pd(nt)-fill),storPlim*(Estor(nt)/storTlim),...
                                                    (Estor(nt))*eta/Dt]);
     fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt)+Pccs1(nt);
     Estor(nt) = Estor(nt)-Pstor(nt)*Dt/eta;
%     end
  end
  if fill < Pd(nt) % fill up from EVs
%     if Ev2g(nt) > 0 % storage not empty
     Pv2g(nt) = min([(Pd(nt)-fill),v2glim*(Ev2g(nt)/v2gTlim),...
                                                      (Ev2g(nt))*eta/Dt]);
     fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt)+Pccs1(nt);
     Ev2g(nt) = Ev2g(nt)-Pv2g(nt)*Dt/eta;
%     end
  end
  if nt>1
   if fill < Pd(nt) % need to increase nuclear gen
    Pn1(nt) = min([nlim,Pn1(nt-1)+nlim*0.02,Pn1(nt-1)+(Pd(nt)-fill)]);
    fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt)+Pccs1(nt);
   end
  end
  
  if fill < Pd(nt) % now need fossil fuels
    Pb1(nt) = min([blim,Pd(nt)-fill]);
    fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt)+Pccs1(nt)+...
                                                               Pb1(nt);
  end
  if fill < Pd(nt)
    Pcc1(nt) = min([cclim,Pd(nt)-fill]);
    fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt)+Pccs1(nt)+...
                                                         Pb1(nt)+Pcc1(nt);
  end
  if fill < Pd(nt) % need to import - last in merit order
    Pdu1(nt) = min([(Pd(nt)-fill),ictlim]);
%     fill = Pn1(nt)+w(nt)+Pv2g(nt)+Pstor(nt)+Pfr1(nt)+Phy1(nt)+Pccs1(nt);
  end
  other(nt) = Pd(nt)-fill;
  
end

Pdu1 = max([Pdu1;other]);

nuccolor = colors(1,:);
coalcolor = colors(2,:);
ccgtcolor = colors(3,:);
bcolor = colors(4,:);
hycolor = colors(5,:);
frcolor = colors(6,:);
wcolor = colors(8,:);
v2gcolor = colors(9,:);
storcolor = colors(10,:);

if plotflag == true
t=t+years(33);
figure('position',[1,60,850,350])
axes('Position', [0.06 0.14 0.42 0.8]);
plot(t,Pd*1e-3,'k')
hold on
grid on
h=area(t,1e-3*[Pn1',Pccs1',w',Phy1',Pfr1',Pb1',...
                          Pcc1',Pdu1'],'linestyle',':','facealpha',0.7);
ylabel('Power (GW)','Interpreter','latex')
h(1).FaceColor = nuccolor;
h(2).FaceColor = coalcolor;
h(3).FaceColor = wcolor;
h(4).FaceColor = hycolor;
h(5).FaceColor = frcolor;
% h(6).FaceColor = v2gcolor;
% h(7).FaceColor = storcolor;
h(6).FaceColor = bcolor;
h(7).FaceColor = ccgtcolor;
h(8).FaceColor = frcolor;
plot(t,Pd*1e-3,'k')
xlim([datetime(2050,5,21)-hours(1),datetime(2050,5,28)+hours(1)])
ylim([0 100])
title('(a)')
axes('Position', [0.555 0.14 0.42 0.8]);
plot(t,Pd*1e-3,'k')
hold on
grid on
h=area(t,1e-3*[Pn1',Pccs1',w',Phy1',Pfr1',Pb1',...
                          Pcc1',Pdu1'],'linestyle',':','facealpha',0.7);
ylabel('Power (GW)','Interpreter','latex')
h(1).FaceColor = nuccolor;
h(2).FaceColor = coalcolor;
h(3).FaceColor = wcolor;
h(4).FaceColor = hycolor;
h(5).FaceColor = frcolor;
% h(6).FaceColor = v2gcolor;
% h(7).FaceColor = storcolor;
h(6).FaceColor = bcolor;
h(7).FaceColor = ccgtcolor;
h(8).FaceColor = frcolor;
plot(t,Pd*1e-3,'k')
l1=legend('demand','nuclear','CCS','wind','other renewable',...
                      'interconnectors','biomass','CCGT');
set(l1,'position',[0.33 0.72 0.1 0.16])                  
xlim([datetime(2050,11,25)-hours(1),datetime(2050,12,2)+hours(1)])
ylim([0 100])
title('(b)')
print -depsc genCR50.eps
end

end

