function [mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du,diags]=...
    hawkes_marg2(Dt,Pd_,wind_,nuc_,bio_,hydro_,dutch_,french_,coal_,ccgt_)
% [mgr_c,mgr_cc,mgr_b,mgr_hy,mgr_fr,mgr_du,diags]=...
% hawkes_marg2(Dt,Pd_,wind_,nuc_,bio_,hydro_,dutch_,french_,coal_,ccgt_)
%     applies adapted Hawkes' method to demand and generation data input,
%     outputs marginal generator response MGR(t) for coal, CCCGT,
%     biomass, hydro, French and Dutch interconnectors.
%     Also diagnostics 'diags', linear fitting R^2 values.

diffPd = 0.5*(diff([(2*Pd_(1)-Pd_(2)) Pd_])+...
               diff([Pd_ (2*Pd_(end)-Pd_(end-1))]));
w = Pd_-wind_;           
diffPdw = 0.5*(diff([(2*w(1)-w(2)) w])+...
                  diff([w (2*w(end)-w(end-1))]));
diffc = 0.5*(diff([(2*coal_(1)-coal_(2)) coal_])+...
                  diff([coal_ (2*coal_(end)-coal_(end-1))]));
diffcc = 0.5*(diff([(2*ccgt_(1)-ccgt_(2)) ccgt_])+...
                  diff([ccgt_ (2*ccgt_(end)-ccgt_(end-1))]));
diffb = 0.5*(diff([(2*bio_(1)-bio_(2)) bio_])+...
                  diff([bio_ (2*bio_(end)-bio_(end-1))]));
diffhy = 0.5*(diff([(2*hydro_(1)-hydro_(2)) hydro_])+...
                  diff([hydro_ (2*hydro_(end)-hydro_(end-1))]));
difffr = 0.5*(diff([(2*french_(1)-french_(2)) french_])+...
                  diff([french_ (2*french_(end)-french_(end-1))]));
diffdu = 0.5*(diff([(2*dutch_(1)-dutch_(2)) dutch_])+...
                  diff([dutch_ (2*dutch_(end)-dutch_(end-1))]));              

opts = optimset('Display','off'); % options for least-squares curve fitting
N = length(diffPd);
bins = [12e3,20e3:2.5e3:53e3,64e3]; % demand bins (MW)
seasons = [0:30:330,365]; % days at month boundaries
Nb = length(bins)-1;
Ns = length(seasons)-1;
masks = zeros(Ns,Nb,N);
masks = (masks>1);

% initialise arrays
% linear fit gradients
m_c = zeros(Ns,Nb);
m_cc = zeros(Ns,Nb);
m_b = zeros(Ns,Nb);
m_hy = zeros(Ns,Nb);
m_fr = zeros(Ns,Nb);
m_du = zeros(Ns,Nb);
% R^2 of linear fittings
rsq_b = zeros(Ns,Nb);
rsq_h = zeros(Ns,Nb);
rsq_c = zeros(Ns,Nb);
rsq_cc = zeros(Ns,Nb);
rsq_fr = zeros(Ns,Nb);
rsq_du = zeros(Ns,Nb);
% Marginal Generator Response
mgr_c = zeros(1,N);
mgr_cc = zeros(1,N);
mgr_b = zeros(1,N);
mgr_hy = zeros(1,N);
mgr_fr = zeros(1,N);
mgr_du = zeros(1,N);

for ns = 1:Ns
  slim1 = seasons(ns)*24/Dt+1;
  slim2 = seasons(ns+1)*24/Dt;
  slim3 = seasons(floor(0.5*(ns-1))*2+1)*24/Dt+1;
  slim4 = seasons(floor(0.5*(ns-1))*2+3)*24/Dt;
 for nbin = 1:Nb
  binlim1 = bins(nbin);
  binlim2 = bins(nbin+1);
  masks(ns,nbin,:) = abs(Pd_-0.5*(binlim1+binlim2))<0.5*(binlim2-binlim1);
  mask2 = masks(ns,nbin,:);
  mask3 = mask2;
  mask2(1:slim1) = false;
  mask2(slim2:end) = false; % mask2 selects data just in month ns
  mask3(1:slim3) = false;
  mask3(slim4:end) = false; % mask3 selects data in current 2-month period
  
 if sum(mask2)>3 % don't bother regressions if <3 data points
  x = diffPd(mask3);
  y = diffc(mask3);  % coal
  [p,~,~,~,stats] = regress(y',[ones(length(x),1),x']);
  m_c(ns,nbin) = p(2);
  rsq_c(ns,nbin) = stats(1);
  
  x = diffPdw(mask2);
  y = diffcc(mask2); % CCGT
  [p,~,~,~,stats] = regress(y',[ones(length(x),1),x']);
  m_cc(ns,nbin) = p(2);
  rsq_cc(ns,nbin) = stats(1);
  
  x = diffPd(mask3);
  y = diffb(mask3);  % biomass
  [p,~,~,~,stats] = regress(y',[ones(length(x),1),x']);
  m_b(ns,nbin) = p(2);
  rsq_b(ns,nbin) = stats(1);
  
  x = diffPd(mask3);
  y = diffhy(mask3); % hydroelectric
  [p,~,~,~,stats] = regress(y',[ones(length(x),1),x']);
  m_hy(ns,nbin) = p(2);
  rsq_h(ns,nbin) = stats(1);
  
  x = diffPd(mask3);
  y = difffr(mask3); % French interconnector
  [p,~,~,~,stats] = regress(y',[ones(length(x),1),x']);
  m_fr(ns,nbin) = p(2);
  rsq_fr(ns,nbin) = stats(1);
  
  x = diffPd(mask3);
  y = diffdu(mask3); % Dutch interconnector
  [p,~,~,~,stats] = regress(y',[ones(length(x),1),x']);
  m_du(ns,nbin) = p(2);
  rsq_du(ns,nbin) = stats(1);
 end
 end
end
% populate MGR(t) with linear fit gradients according to month/demand bins
for nt = 1:N
  if nt<24*seasons(2)/Dt
    ns = 1;
  elseif nt<24*seasons(3)/Dt
    ns = 2;
  elseif nt<24*seasons(4)/Dt
    ns = 3;
  elseif nt<24*seasons(5)/Dt
    ns = 4;
  elseif nt<24*seasons(6)/Dt
    ns = 5;
  elseif nt<24*seasons(7)/Dt
    ns = 6;
  elseif nt<24*seasons(8)/Dt
    ns = 7;
  elseif nt<24*seasons(9)/Dt
    ns = 8;
  elseif nt<24*seasons(10)/Dt
    ns = 9;
  elseif nt<24*seasons(11)/Dt
    ns = 10;
  elseif nt<24*seasons(12)/Dt
    ns = 11;
  else
    ns = 12;
  end
  [~,i] = min(abs(Pd_(nt)-bins));
  if Pd_(nt) < bins(i)
    i = i-1;
  end
  mgr_c(nt) = m_c(ns,i);
  mgr_cc(nt) = m_cc(ns,i);
  mgr_b(nt) = m_b(ns,i);
  mgr_hy(nt) = m_hy(ns,i);
  mgr_fr(nt) = m_fr(ns,i);
  mgr_du(nt) = m_du(ns,i);
  
end

% diagnostics - R-sq for biomass, hydro, coal, CCGT, french, dutch
diags = [{rsq_b};
         {rsq_h};
         {rsq_c};
         {rsq_cc};
         {rsq_fr};
         {rsq_du}];

end

